//
//  CitiesTableViewController.swift
//  Capitales
//
//  Created by Santiago Pavón Gómez on 11/12/15.
//  Copyright © 2015 UPM. All rights reserved.
//

import UIKit
import MapKit

class CitiesTableViewController: UITableViewController {

    // URLs Base de la API y de los iconos.
    fileprivate let API_OPEN_WEATHER_URL = "http://api.openweathermap.org/data/2.5/weather"
    fileprivate let ICON_OPEN_WEATHER_URL = "http://openweathermap.org/img/w"
    
    // Hay que crearse una cuenta en http://api.openweathermap.org para poder acceder al API.
    fileprivate let APPID = ""
    // private let APPID = ""
    
    // Array donde salvare las ciudades a mostrar
    fileprivate var cities: [String] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        loadCities()
    }

    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        checkAPIID()
    }
    
    
    // Comprobar hemos puesto el valor de APIID
    fileprivate func checkAPIID() {
        if APPID == "" {
            let alert = UIAlertController(title: "Regístrese", message: "Para utilizar esta aplicación debe registrarse en http://openweathermap.org y obtener una clase de acceso al API", preferredStyle: .alert)
            
            alert.addAction(UIAlertAction(
                title: "Vale",
                style: UIAlertActionStyle.default) {(_: UIAlertAction) -> Void in
                    self.dismiss(animated: true, completion: nil)
                })
            
            present(alert, animated: true)
        }
    }
    
    
    /** 
     * Leer el fichero con las ciudades a mostrar
     */
    fileprivate func loadCities() {
        
        print("Cargando Ciudades ...")
        
        if let path = Bundle.main.path(forResource: "cities", ofType: "plist") {
            if let cities = NSArray(contentsOfFile: path) as? [String] {
                self.cities = cities
                tableView.reloadData()
            }
        }
    }

    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return cities.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "City Cell", for: indexPath) as! CityTableViewCell

        let cityName = cities[indexPath.row]
        
        cell.cityLabel.text = cityName
        
        cell.mapView.alpha = 0
        
        configureCellForCityNamed(cityName, atIndexPath: indexPath)

        return cell
    }

    fileprivate func configureCellForCityNamed(_ name: String, atIndexPath ip: IndexPath) {
        
        var queue = DispatchQueue(label: "cell queue", attributes: [])
        queue.async {
            
            self.netActivityVisible = true
            defer {
                self.netActivityVisible = false
            }
            
            guard let query = "q=\(name)&units=metric&lang=sp&APPID=\(self.APPID)".addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) else {
                print("Error escapando URL")
                return
            }
            
            guard let url = URL(string: "\(self.API_OPEN_WEATHER_URL)?\(query)") else {
                print("Error: URL mal formada")
                return
            }
            
            guard let data = try? Data(contentsOf: url) else {
                print("Error: no se han podido descargar los datos.")
                return
            }
            
            do {
                if let dic = try JSONSerialization.jsonObject(with: data, options: JSONSerialization.ReadingOptions()) as? [String:AnyObject] {
                    
                    //print(dic)
                    
                    if let cod = dic["cod"] as? Int, cod == 200 {
                        self.updateCellAtIndexPath(ip, withDictionary: dic)
                    } else {
                        let msg = dic["message"] as? String ?? "Error: Desconocido"
                        print(msg)
                    }
                } else {
                    print("Los datos recibidos no tienen un formato inesperado.")
                
                }
            } catch let err  {
                print("No puedo sacar el JSON:", err.localizedDescription)
                print(String(data: data, encoding: String.Encoding.utf8) ?? "Ni idea de lo que esta pasando.")
            }
        }
    }
    
    fileprivate func updateCellAtIndexPath(_ ip: IndexPath, withDictionary dic: [String:AnyObject]) {
        
        DispatchQueue.main.async {
            
            if let cell = self.tableView.cellForRow(at: ip) as? CityTableViewCell {
                
                if let cityName = dic["name"] as? String {
                    cell.cityLabel.text = cityName
                }
                
                let main = dic["main"] as! [String:AnyObject]
                let sys = dic["sys"] as! [String:AnyObject]
                let weather = dic["weather"] as! [[String:AnyObject]]
                let coord = dic["coord"] as! [String:AnyObject]
                
                if let latitude = coord["lat"] as? Float {
                    if let longitude = coord["lon"] as? Float {
                        
                        let center = CLLocationCoordinate2D(latitude: CLLocationDegrees(latitude), longitude: CLLocationDegrees(longitude))
                        let span = MKCoordinateSpanMake(0.3, 0.3)
                        let reg =  MKCoordinateRegionMake(center, span)
                        
                        cell.mapView.setRegion(reg, animated: false)
                        cell.mapView.mapType = .hybrid
                        cell.mapView.isUserInteractionEnabled = false
                        
                        cell.mapView.alpha = 1
                    }
                }
                
                if let tempMin = main["temp_min"] as? Float {
                    cell.minTempLabel.text = "\(tempMin)ºC"
                }
                if let temp = main["temp"] as? Float {
                    cell.tempLabel.text = "\(temp)ºC"
                }
                if let tempMax = main["temp_max"] as? Float {
                    cell.maxTempLabel.text = "\(tempMax)ºC"
                }
                
                let dateFormatter = DateFormatter()
                dateFormatter.dateFormat = "HH:mm"
                
                if let ndate = sys["sunrise"] as? Double {
                    let date = Date(timeIntervalSince1970: ndate)
                    cell.sunriseLabel.text = dateFormatter.string(from: date)
                }
                if let ndate = sys["sunset"] as? Double {
                    let date = Date(timeIntervalSince1970: ndate)
                    cell.sunsetLabel.text = dateFormatter.string(from: date)
                }
                
                for pos in 0  ..< min(3, weather.count) {
                    if let icon = weather[pos]["icon"] as? String {
                        self.setIcon(icon, forWeatherPosition: pos, atIndexPath: ip)
                    }
                }
            }
        }
    }
   
    /**
     Descargar un icono y ponerlo en la celda indicada, en una de las casillas reservadas para ello.
     */
    fileprivate func setIcon(_ iconName: String, forWeatherPosition wp: Int, atIndexPath ip: IndexPath) {
        
        let queue = DispatchQueue(label: "icon queue", attributes: [])
        queue.async {
            
            if let icon = self.getIconNamed(iconName) {
                
                DispatchQueue.main.async {
                    
                    if let cell = self.tableView.cellForRow(at: ip) as? CityTableViewCell {
                        
                        switch wp {
                        case 0:
                            cell.weather1ImageView.image = icon
                        case 1:
                            cell.weather0ImageView.image = icon
                        case 2:
                            cell.weather2ImageView.image = icon
                        default:
                            break
                        }
                    }
                }
            }
        }
    }
    
    /**
     Cache donde guardo los iconos que ya me he bajado.
     */
    fileprivate var iconsCache: [String:UIImage] = [:]
    
    /**
     Descargar un icono representando el tiempo
     */
    fileprivate func getIconNamed(_ name: String) -> UIImage? {
        
        if let img = iconsCache[name] { // Ya lo he bajado antes.
            return img
        }
        
        netActivityVisible = true
        defer {
            netActivityVisible = false
        }

        guard let url = URL(string: "\(ICON_OPEN_WEATHER_URL)/\(name).png") else {
            print("Error: URL mal formada")
            return nil
        }
        
        guard let data = try? Data(contentsOf: url) else {
            print("Error: no se han podido descargar el icono.")
            return nil
        }
        
        guard let img = UIImage(data: data) else {
            print("Los datos recibidos no tienen un formato inesperado.")
            return nil
        }
        
        // Guardarlo en la cache
        iconsCache[name] = img
        
        return img
    }

    
    // MARK: - TableView Delegado

    
    override func tableView(_ tableView: UITableView, willSelectRowAt indexPath: IndexPath) -> IndexPath? {

        // Devolver nil para que no se seleccione la celda.
        return nil
    }
    
    
    
    // MARK: - Network Activity
  
    /**
    Contador interno usado para saber cuantas veces hemos ejecutado:
    
    * UIApplication.sharedApplication().networkActivityIndicatorVisible = true
    
    Se activa el indicador de actividad en la red si este contador sea mayor que cero.
    
    Este contador no debe usarse directamente en el programa. Debe usarse la propiedad netActivityVisible.
    */
    fileprivate var netActivityCredit = 0 {
        didSet {
            UIApplication.shared.isNetworkActivityIndicatorVisible = netActivityCredit > 0
        }
    }
    
    /**
     Activar o desactivar el indicador de actividad de red.
     
     Se ejecuta en el Main Thread.
     */
    fileprivate var netActivityVisible: Bool {
        get {
            var res = false
            DispatchQueue.main.sync {
                res  = self.netActivityCredit > 0
            }
            return res
        }
        set {
            DispatchQueue.main.sync {
                if (newValue) {
                    self.netActivityCredit += 1
                } else {
                    self.netActivityCredit -= 1
                }
            }
        }
    }
    
}
